#include <stdio.h>
#include <stdlib.h>
#include "imsls.h"

int main (){                                
   int i, j, k, m, n_patterns, n_var;
   int n_cat         =9;    /* 9 nominal input attributes */
   int n_categorical =27;  /* 9 Encoded = 27 categorical inputs */
   int n_classes     =2;   /* positive or negative */
   float *classProb;
   float *trainStats;
   int *predictedClass;
   int classification[958];
   float *inputData;  
   int *categoricalAtt, *nomTempIn, *nomTempOut, nClass;
   Imsls_f_NN_Network *network;

   /* get tic tac toe data */ 
   inputData = imsls_f_data_sets (10, 
      IMSLS_N_OBSERVATIONS, &n_patterns, 
      IMSLS_N_VARIABLES, &n_var, 0);

   printf("\n\n");
   printf("*******************************************************\n");
   printf("* TIC-TAC-TOE BINARY CLASSIFICATION EXAMPLE           *\n");
   printf("*******************************************************\n");
   /* allocate memory for categoricalATT array */
   categoricalAtt    = (int*)malloc(958*n_categorical*sizeof(int));
   /* populate categorical Att from catAtt using binary encoding */
   nomTempIn  = (int*) malloc(n_patterns*sizeof(int));
   m=0;

   for(i=0; i<n_cat; i++)
   {
      for(j=0; j<n_patterns; j++)
         nomTempIn[j] = (int) inputData[j*n_var+i]+1; 
      nomTempOut = 
         imsls_unsupervised_nominal_filter(n_patterns, &nClass, 
         nomTempIn, 0);
      for(k=0; k<nClass; k++)
      {
         for(j=0; j<n_patterns; j++)
            categoricalAtt[j*n_categorical+m] = nomTempOut[j*nClass+k]; 
         m++;
      }
      free(nomTempOut);
   }
   free(nomTempIn);

   /* Setup the classification array, classification[] */
   for(i=0; i < n_patterns; i++)
      classification[i] = (int)inputData[(i*n_var)+n_var-1];

   network = imsls_f_mlff_network_init(27, 1);
   imsls_f_mlff_network(network, IMSLS_CREATE_HIDDEN_LAYER, 8,
      IMSLS_LINK_ALL, 0);

   imsls_random_seed_set(5555);
   /* Train Classification Network */
   trainStats = imsls_f_mlff_classification_trainer (network,
      n_patterns, n_categorical, 0, classification, categoricalAtt,
      NULL, IMSLS_STAGE_I, 30, n_patterns, IMSLS_NO_STAGE_II,
      IMSLS_LOGISTIC_TABLE,
      IMSLS_WEIGHT_INITIALIZATION_METHOD, IMSLS_EQUAL, 0);

   /* 
   * Use pattern classification routine to classify training patterns 
   * using trained network.  This will reproduce the results returned 
   * in predicted_class[]
   */
   classProb = imsls_f_mlff_pattern_classification (network, 
       n_patterns, n_categorical, 0, categoricalAtt, NULL,
       IMSLS_LOGISTIC_TABLE, 
       IMSLS_PREDICTED_CLASS, &predictedClass, 0);

   /* Printing Classification Predictions */
   printf("*******************************************************\n");
   printf("Classification Minimum Cross-Entropy Error: %f\n", 
      trainStats[0]);
   printf("Classification Error Rate: %f \n", trainStats[5]);
   printf("*******************************************************\n");
   printf("\nPRINTING FIRST TEN PREDICTIONS FOR EACH TARGET CLASS\n");
   printf("*******************************************************\n");
   printf("        |TARGET|PREDICTED|             |              *\n");
   printf("PATTERN |CLASS |  CLASS  | P(class=0)  |  P(class=1)  *\n");
   printf("*******************************************************\n");
   for(k=0; k<2; k++){
      for(i=k*627; i<k*627+10; i++){
         printf("  %d\t|  %d   |    %d    |  ", i+1, 
            classification[i], predictedClass[i]);
         printf("%f   |   %f \n",
            classProb[i*n_classes], classProb[i*n_classes+1]);
      }
      printf("\n");
   }
   k=0;
   for(i=0; i<n_patterns; i++) 
      if(classification[i] != predictedClass[i]) k++;
   if(k==0) {
      printf("All %d predicted classifications agree ", n_patterns);
      printf("with target classifications\n");
   }
}
